package io.gitlab.jfronny.quickmeth.mixin;

import io.gitlab.jfronny.quickmeth.MathUtil;
import net.minecraft.util.math.noise.PerlinNoiseSampler;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.ModifyVariable;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfoReturnable;

@Mixin(PerlinNoiseSampler.class)
public abstract class MixinPerlinNoiseSampler {
    @ModifyVariable(method = "sample(IIIDDDD)D", at = @At("HEAD"), argsOnly = true, ordinal = 0)
    private double sampleAdjustX(double localX) {
        return MathUtil.BoxedInvert(localX);
    }

    @ModifyVariable(method = "sampleDerivative(IIIDDD[D)D", at = @At("HEAD"), argsOnly = true, ordinal = 0)
    private double derivAdjustX(double localX) {
        return MathUtil.BoxedInvert(localX);
    }

    //Once again, the y parameter is ignored because mixin

    @ModifyVariable(method = "sample(IIIDDDD)D", at = @At("HEAD"), argsOnly = true, ordinal = 2)
    private double sampleAdjustZ(double localZ) {
        return MathUtil.BoxedInvert(localZ);
    }

    @ModifyVariable(method = "sampleDerivative(IIIDDD[D)D", at = @At("HEAD"), argsOnly = true, ordinal = 2)
    private double derivAdjustZ(double localZ) {
        return MathUtil.BoxedInvert(localZ);
    }

    @Inject(method = "sample(IIIDDDD)D", at = @At("TAIL"), cancellable = true)
    private void sampleAsjustResult(CallbackInfoReturnable<Double> ret) {
        ret.setReturnValue(MathUtil.BoxedInvert(ret.getReturnValue()));
    }

    @Inject(method = "sampleDerivative(IIIDDD[D)D", at = @At("TAIL"), cancellable = true)
    private void derivAsjustResult(CallbackInfoReturnable<Double> ret) {
        ret.setReturnValue(MathUtil.BoxedInvert(ret.getReturnValue()));
    }
}
